@@ -1,7 +1,10 @@
 import logging
-from typing import List
+from typing import List, Optional
 
-from .exceptions import XIVAPIBadRequest, XIVAPIForbidden, XIVAPINotFound, XIVAPIServiceUnavailable, XIVAPIInvalidLanguage, XIVAPIError, XIVAPIInvalidIndex, XIVAPIInvalidColumns
+from aiohttp import ClientSession
+
+from .exceptions import XIVAPIBadRequest, XIVAPIForbidden, XIVAPINotFound, XIVAPIServiceUnavailable, \
+    XIVAPIInvalidLanguage, XIVAPIError, XIVAPIInvalidIndex, XIVAPIInvalidColumns, XIVAPIInvalidAlgo
 from .decorators import timed
 from .models import Filter, Sort
 
@@ -13,18 +16,31 @@ class XIVAPIClient:
     Asynchronous client for accessing XIVAPI's endpoints.
     Parameters
     ------------
-    session: aiohttp.ClientSession()
-        The aiohttp session used with which to make http requests
     api_key: str
         The API key used for identifying your application with XIVAPI.com.
+    session: Optional[ClientSession]
+        Optionally include your aiohttp session
     """
+    base_url = "https://xivapi.com"
+    languages = ["en", "fr", "de", "ja"]
 
-    def __init__(self, session, api_key):
-        self.session = session
+    def __init__(self, api_key: str, session: Optional[ClientSession] = None) -> None:
         self.api_key = api_key
+        self._session = session
 
         self.base_url = "https://xivapi.com"
         self.languages = ["en", "fr", "de", "ja"]
+        self.string_algos = [
+            "custom", "wildcard", "wildcard_plus", "fuzzy", "term", "prefix", "match", "match_phrase",
+            "match_phrase_prefix", "multi_match", "query_string"
+        ]
+
+
+    @property
+    def session(self) -> ClientSession:
+        if self._session is None or self._session.closed:
+            self._session = ClientSession()
+        return self._session
 
 
     @timed
@@ -242,7 +258,7 @@ class XIVAPIClient:
 
 
     @timed
-    async def index_search(self, name, indexes=(), columns=(), filters: List[Filter]=(), sort: Sort=None, page=1, language="en"):
+    async def index_search(self, name, indexes=(), columns=(), filters: List[Filter]=(), sort: Sort=None, page=1, language="en", string_algo="match"):
         """|coro|
         Search for data from on specific indexes.
         Parameters
@@ -265,6 +281,10 @@ class XIVAPIClient:
         Optional[language: str]
             The two character length language code that indicates the language to return the response in. Defaults to English (en).
             Valid values are "en", "fr", "de" & "ja"
+        Optional[string_algo: str]
+            The search algorithm to use for string matching (default = "match")
+            Valid values are "custom", "wildcard", "wildcard_plus", "fuzzy", "term", "prefix", "match", "match_phrase",
+            "match_phrase_prefix", "multi_match", "query_string"
         """
 
         if len(indexes) == 0:
@@ -276,14 +296,17 @@ class XIVAPIClient:
         if len(columns) == 0:
             raise XIVAPIInvalidColumns("Please specify at least one column to return in the resulting data.")
 
+        if string_algo not in self.string_algos:
+            raise XIVAPIInvalidAlgo(f'"{string_algo}" is not a supported string_algo for XIVAPI')
+
         body = {
             "indexes": ",".join(list(set(indexes))),
             "columns": "ID",
-            "body" : {
+            "body": {
                 "query": {
                     "bool": {
                         "should": [{
-                            "match": {
+                            string_algo: {
                                 "NameCombined_en": {
                                     "query": name,
                                     "fuzziness": "AUTO",
@@ -292,7 +315,7 @@ class XIVAPIClient:
                                 }
                             }
                         }, {
-                            "match": {
+                            string_algo: {
                                 "NameCombined_de": {
                                     "query": name,
                                     "fuzziness": "AUTO",
@@ -301,7 +324,7 @@ class XIVAPIClient:
                                 }
                             }
                         }, {
-                            "match": {
+                            string_algo: {
                                 "NameCombined_fr": {
                                     "query": name,
                                     "fuzziness": "AUTO",
@@ -310,7 +333,7 @@ class XIVAPIClient:
                                 }
                             }
                         }, {
-                            "match": {
+                            string_algo: {
                                 "NameCombined_ja": {
                                     "query": name,
                                     "fuzziness": "AUTO",
@@ -385,7 +408,6 @@ class XIVAPIClient:
         async with self.session.get(url, params=params) as response:
             return await self.process_response(response)
 
-
     @timed
     async def lore_search(self, query, language="en"):
         """|coro|
